import { CompositeLayer } from '@deck.gl/core';
import { Colors, DiffColors } from './colors';
import { LayerProps } from './LayerProps';
import { Flow, FlowAccessor, FlowLayerPickingInfo, Location, LocationAccessor, Locations, PickingHandler } from './types';
export interface BasicProps extends LayerProps {
    locations: Locations;
    flows: Flow[];
    diffMode?: boolean;
    animate?: boolean;
    animationCurrentTime?: number;
    animationTailLength?: number;
    colors?: Colors | DiffColors;
    getLocationId?: LocationAccessor<string>;
    getLocationCentroid?: LocationAccessor<[number, number]>;
    getLocationTotalIn?: LocationAccessor<number>;
    getLocationTotalOut?: LocationAccessor<number>;
    getLocationTotalWithin?: LocationAccessor<number>;
    getFlowOriginId?: FlowAccessor<string>;
    getFlowDestId?: FlowAccessor<string>;
    getFlowMagnitude?: FlowAccessor<number>;
    getAnimatedFlowLineStaggering?: FlowAccessor<number>;
    getFlowColor?: FlowAccessor<string | undefined>;
    maxFlowThickness?: number;
    flowMagnitudeExtent?: [number, number];
    locationTotalsExtent?: [number, number];
    maxLocationCircleSize?: number;
    minPickableFlowThickness?: number;
    showTotals?: boolean;
    showLocationAreas?: boolean;
    showOnlyTopFlows?: number;
    selectedLocationIds?: string[];
    highlightedLocationId?: string;
    highlightedLocationAreaId?: string;
    highlightedFlow?: Flow;
    outlineThickness?: number;
    updateTriggers?: {
        getFlowLinesSourcePosition?: any;
        getFlowLinesTargetPosition?: any;
        getFlowLinesThickness?: any;
        getFlowLinesEndpointOffsets?: any;
        getFlowLinesColor?: any;
        getCirclesRadius?: any;
        getCirclesColor?: any;
        getLocationAreasFillColor?: any;
        getLocationAreasLineColor?: any;
    };
}
export interface Props extends BasicProps {
    id: string;
    onClick?: PickingHandler<FlowLayerPickingInfo>;
    onHover?: PickingHandler<FlowLayerPickingInfo>;
}
export default class FlowMapLayer extends CompositeLayer {
    static layerName: string;
    static defaultProps: {
        getLocationId: {
            type: string;
            value: (l: Location) => any;
        };
        getLocationCentroid: {
            type: string;
            value: (l: Location) => any;
        };
        getFlowOriginId: {
            type: string;
            value: (f: Flow) => any;
        };
        getFlowDestId: {
            type: string;
            value: (f: Flow) => any;
        };
        getFlowMagnitude: {
            type: string;
            value: (f: Flow) => any;
        };
        showTotals: boolean;
        maxLocationCircleSize: number;
        outlineThickness: number;
        showLocationAreas: boolean;
        animationTailLength: number;
    };
    props: Props;
    constructor(props: Props);
    initializeState(): void;
    updateState(params: any): void;
    getPickingInfo(params: any): FlowLayerPickingInfo;
    renderLayers(): any[];
    private getLocationAreasLayer;
    private getHighlightedLocationAreasLayer;
    private getFlowLinesLayer;
    private getLocationCirclesLayer;
}
//# sourceMappingURL=FlowMapLayer.d.ts.map