export declare type ColorScale = (value: number) => RGBA;
export declare type RGBA = [number, number, number, number];
export interface FlowColors {
    scheme?: string[];
    highlighted?: string;
}
export interface LocationCircleColors {
    inner?: string;
    outgoing?: string;
    incoming?: string;
    highlighted?: string;
}
export interface LocationAreaColors {
    outline?: string;
    normal?: string;
    selected?: string;
    highlighted?: string;
    connected?: string;
}
export interface BaseColors {
    darkMode?: boolean;
    locationAreas?: LocationAreaColors;
    dimmedOpacity?: number;
    outlineColor?: string;
}
export interface Colors extends BaseColors {
    flows?: FlowColors;
    locationCircles?: LocationCircleColors;
}
export interface FlowAndCircleColors {
    flows?: FlowColors;
    locationCircles?: LocationCircleColors;
}
export interface DiffColors extends BaseColors {
    positive?: FlowAndCircleColors;
    negative?: FlowAndCircleColors;
}
export interface FlowColorsRGBA {
    scheme: string[];
    highlighted: RGBA;
}
export interface LocationCircleColorsRGBA {
    inner: RGBA;
    outgoing: RGBA;
    incoming: RGBA;
    highlighted: RGBA;
}
export interface LocationAreaColorsRGBA {
    outline: RGBA;
    normal: RGBA;
    selected: RGBA;
    highlighted: RGBA;
    connected: RGBA;
}
export interface BaseColorsRGBA {
    darkMode: boolean;
    locationAreas: LocationAreaColorsRGBA;
    dimmedOpacity: number;
    outlineColor: RGBA;
}
export interface ColorsRGBA extends BaseColorsRGBA {
    flows: FlowColorsRGBA;
    locationCircles: LocationCircleColorsRGBA;
}
export interface FlowAndCircleColorsRGBA {
    flows: FlowColorsRGBA;
    locationCircles: LocationCircleColorsRGBA;
}
export interface DiffColorsRGBA extends BaseColorsRGBA {
    positive: FlowAndCircleColorsRGBA;
    negative: FlowAndCircleColorsRGBA;
}
export declare function isDiffColors(colors: DiffColors | Colors): colors is DiffColors;
export declare function isDiffColorsRGBA(colors: DiffColorsRGBA | ColorsRGBA): colors is DiffColorsRGBA;
export declare function getColorsRGBA(colors: Colors | undefined): ColorsRGBA;
export declare function getDiffColorsRGBA(colors: DiffColors | undefined): DiffColorsRGBA;
export declare function colorAsRgba(color: string): RGBA;
export declare function rgbaAsString(color: RGBA): string;
export declare function opacifyHex(hexCode: string, opacity: number): string;
export declare function opacityFloatToInteger(opacity: number): number;
export declare function getDimmedCircleOutlineColor(outlineColor: RGBA, opacity: number): RGBA;
export declare function getDimmedCircleColor(color: RGBA, opacity: number): RGBA;
export declare function getDimmedColor(color: RGBA, opacity?: number): RGBA;
export declare function createFlowColorScale(domain: [number, number], scheme: string[], animate: boolean | undefined): ColorScale;
//# sourceMappingURL=colors.d.ts.map