"use strict";
/*
 * Copyright 2019 Teralytics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * This file was modified by Teralytics. The original file is licenced under:
 *
 * Copyright (c) 2015-2017 Uber Technologies, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var core_1 = require("@deck.gl/core");
var constants_1 = __importDefault(require("@luma.gl/constants"));
var core_2 = require("@luma.gl/core");
var AnimatedFlowLinesLayerFragment_glsl_1 = __importDefault(require("./AnimatedFlowLinesLayerFragment.glsl"));
var AnimatedFlowLinesLayerVertex_glsl_1 = __importDefault(require("./AnimatedFlowLinesLayerVertex.glsl"));
var DEFAULT_COLOR = [0, 132, 193, 255];
var AnimatedFlowLinesLayer = /** @class */ (function (_super) {
    __extends(AnimatedFlowLinesLayer, _super);
    function AnimatedFlowLinesLayer(props) {
        return _super.call(this, props) || this;
    }
    AnimatedFlowLinesLayer.prototype.getShaders = function () {
        return _super.prototype.getShaders.call(this, {
            vs: AnimatedFlowLinesLayerVertex_glsl_1.default,
            fs: AnimatedFlowLinesLayerFragment_glsl_1.default,
            modules: [core_1.project32, core_1.picking],
            shaderCache: this.context.shaderCache,
        });
    };
    AnimatedFlowLinesLayer.prototype.initializeState = function () {
        var attributeManager = this.getAttributeManager();
        /* eslint-disable max-len */
        attributeManager.addInstanced({
            instanceSourcePositions: {
                size: 3,
                transition: true,
                accessor: 'getSourcePosition',
            },
            instanceTargetPositions: {
                size: 3,
                transition: true,
                accessor: 'getTargetPosition',
            },
            instanceColors: {
                size: 4,
                type: constants_1.default.UNSIGNED_BYTE,
                transition: true,
                accessor: 'getColor',
                defaultValue: [0, 0, 0, 255],
            },
            instanceWidths: {
                size: 1,
                transition: true,
                accessor: 'getThickness',
                defaultValue: 1,
            },
            instanceStaggering: {
                accessor: 'getStaggering',
                size: 1,
                transition: false,
            },
            instancePickable: {
                accessor: 'getPickable',
                size: 1,
                transition: false,
            },
        });
        /* eslint-enable max-len */
    };
    AnimatedFlowLinesLayer.prototype.updateState = function (_a) {
        var props = _a.props, oldProps = _a.oldProps, changeFlags = _a.changeFlags;
        _super.prototype.updateState.call(this, { props: props, oldProps: oldProps, changeFlags: changeFlags });
        if (changeFlags.extensionsChanged) {
            var gl = this.context.gl;
            if (this.state.model) {
                this.state.model.delete();
            }
            this.setState({ model: this._getModel(gl) });
            this.getAttributeManager().invalidateAll();
        }
    };
    AnimatedFlowLinesLayer.prototype.draw = function (_a) {
        var uniforms = _a.uniforms;
        var _b = this.props, currentTime = _b.currentTime, thicknessUnit = _b.thicknessUnit, animationTailLength = _b.animationTailLength;
        this.state.model
            .setUniforms(__assign(__assign({}, uniforms), { thicknessUnit: thicknessUnit * 4, animationTailLength: animationTailLength,
            currentTime: currentTime }))
            .draw();
    };
    AnimatedFlowLinesLayer.prototype._getModel = function (gl) {
        /*
         *  (0, -1)-------------_(1, -1)
         *       |          _,-"  |
         *       o      _,-"      o
         *       |  _,-"          |
         *   (0, 1)"-------------(1, 1)
         */
        var positions = [0, -1, 0, 0, 1, 0, 1, -1, 0, 1, 1, 0];
        return new core_2.Model(gl, Object.assign({}, this.getShaders(), {
            id: this.props.id,
            geometry: new core_2.Geometry({
                drawMode: constants_1.default.TRIANGLE_STRIP,
                attributes: {
                    positions: new Float32Array(positions),
                },
            }),
            isInstanced: true,
        }));
    };
    AnimatedFlowLinesLayer.defaultProps = {
        currentTime: 0,
        animationTailLength: 0.7,
        getSourcePosition: { type: 'accessor', value: function (d) { return d.sourcePosition; } },
        getTargetPosition: { type: 'accessor', value: function (d) { return d.targetPosition; } },
        getPickable: { type: 'accessor', value: function (d) { return 1.0; } },
        getStaggering: { type: 'accessor', value: function (d, _a) {
                var index = _a.index;
                return Math.random();
            } },
        getColor: { type: 'accessor', value: DEFAULT_COLOR },
        getThickness: { type: 'accessor', value: 1 },
        thicknessUnit: 15 * 2,
        parameters: {
            depthTest: false,
        },
    };
    return AnimatedFlowLinesLayer;
}(core_1.Layer));
exports.default = AnimatedFlowLinesLayer;
//# sourceMappingURL=AnimatedFlowLinesLayer.js.map