"use strict";
/*
 * Copyright 2018 Teralytics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@deck.gl/core");
var constants_1 = __importDefault(require("@luma.gl/constants"));
var core_2 = require("@luma.gl/core");
var FlowCirclesLayerFragment_glsl_1 = __importDefault(require("./FlowCirclesLayerFragment.glsl"));
var FlowCirclesLayerVertex_glsl_1 = __importDefault(require("./FlowCirclesLayerVertex.glsl"));
var DEFAULT_COLOR = [0, 0, 0, 255];
var FlowCirclesLayer = /** @class */ (function (_super) {
    __extends(FlowCirclesLayer, _super);
    function FlowCirclesLayer(props) {
        return _super.call(this, props) || this;
    }
    FlowCirclesLayer.prototype.getShaders = function () {
        return _super.prototype.getShaders.call(this, {
            vs: FlowCirclesLayerVertex_glsl_1.default,
            fs: FlowCirclesLayerFragment_glsl_1.default,
            modules: [core_1.project32, core_1.picking],
        });
    };
    FlowCirclesLayer.prototype.initializeState = function () {
        this.getAttributeManager().addInstanced({
            instancePositions: {
                size: 3,
                type: constants_1.default.DOUBLE,
                fp64: this.use64bitPositions(),
                transition: true,
                accessor: 'getPosition',
            },
            instanceRadius: {
                size: 1,
                transition: true,
                accessor: 'getRadius',
                defaultValue: 1,
            },
            instanceColors: {
                size: 4,
                transition: true,
                type: constants_1.default.UNSIGNED_BYTE,
                accessor: 'getColor',
                defaultValue: DEFAULT_COLOR,
            },
        });
    };
    FlowCirclesLayer.prototype.updateState = function (_a) {
        var props = _a.props, oldProps = _a.oldProps, changeFlags = _a.changeFlags;
        _super.prototype.updateState.call(this, { props: props, oldProps: oldProps, changeFlags: changeFlags });
        if (changeFlags.extensionsChanged) {
            var gl = this.context.gl;
            if (this.state.model) {
                this.state.model.delete();
            }
            this.setState({ model: this._getModel(gl) });
            this.getAttributeManager().invalidateAll();
        }
    };
    FlowCirclesLayer.prototype.draw = function (_a) {
        var uniforms = _a.uniforms;
        this.state.model.setUniforms(uniforms).draw();
    };
    FlowCirclesLayer.prototype._getModel = function (gl) {
        // a square that minimally cover the unit circle
        var positions = [-1, -1, 0, 1, -1, 0, 1, 1, 0, -1, 1, 0];
        return new core_2.Model(gl, Object.assign(this.getShaders(), {
            id: this.props.id,
            geometry: new core_2.Geometry({
                drawMode: constants_1.default.TRIANGLE_FAN,
                vertexCount: 4,
                attributes: {
                    positions: { size: 3, value: new Float32Array(positions) },
                },
            }),
            isInstanced: true,
        }));
    };
    FlowCirclesLayer.layerName = 'FlowCirclesLayer';
    FlowCirclesLayer.defaultProps = {
        getColor: { type: 'accessor', value: DEFAULT_COLOR },
        getPosition: { type: 'accessor', value: function (d) { return d.position; } },
        getRadius: { type: 'accessor', value: 1 },
        parameters: {
            depthTest: false,
        },
    };
    return FlowCirclesLayer;
}(core_1.Layer));
exports.default = FlowCirclesLayer;
//# sourceMappingURL=FlowCirclesLayer.js.map