"use strict";
/*
 * Copyright 2018 Teralytics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@deck.gl/core");
var layers_1 = require("@deck.gl/layers");
var AnimatedFlowLinesLayer_1 = __importDefault(require("./AnimatedFlowLinesLayer/AnimatedFlowLinesLayer"));
var FlowCirclesLayer_1 = __importDefault(require("./FlowCirclesLayer/FlowCirclesLayer"));
var FlowLinesLayer_1 = __importDefault(require("./FlowLinesLayer/FlowLinesLayer"));
var Selectors_1 = __importDefault(require("./Selectors"));
var types_1 = require("./types");
var LayerKind;
(function (LayerKind) {
    LayerKind["LOCATIONS"] = "LOCATIONS";
    LayerKind["LOCATION_AREAS"] = "LOCATION_AREAS";
    LayerKind["LOCATION_AREAS_OUTLINES"] = "LOCATION_AREAS_OUTLINES";
    LayerKind["LOCATION_AREAS_SELECTED_AND_HIGHLIGHTED"] = "LOCATION_AREAS_SELECTED_AND_HIGHLIGHTED";
    LayerKind["FLOWS"] = "FLOWS";
    LayerKind["LOCATIONS_HIGHLIGHTED"] = "LOCATIONS_HIGHLIGHTED";
    LayerKind["FLOWS_HIGHLIGHTED"] = "FLOWS_HIGHLIGHTED";
})(LayerKind || (LayerKind = {}));
var LAYER_ID_SEPARATOR = ':::';
function getLayerId(baseLayerId, layerKind) {
    return "" + baseLayerId + LAYER_ID_SEPARATOR + layerKind.valueOf();
}
function getLayerKind(id) {
    var kind = id.substr(id.lastIndexOf(LAYER_ID_SEPARATOR) + LAYER_ID_SEPARATOR.length);
    return LayerKind[kind];
}
function getPickType(_a) {
    var id = _a.id;
    switch (getLayerKind(id)) {
        case LayerKind.FLOWS:
        case LayerKind.FLOWS_HIGHLIGHTED:
            return types_1.PickingType.FLOW;
        case LayerKind.LOCATIONS:
        case LayerKind.LOCATIONS_HIGHLIGHTED:
            return types_1.PickingType.LOCATION;
        case LayerKind.LOCATION_AREAS:
            return types_1.PickingType.LOCATION_AREA;
        default:
            return undefined;
    }
}
var FlowMapLayer = /** @class */ (function (_super) {
    __extends(FlowMapLayer, _super);
    function FlowMapLayer(props) {
        return _super.call(this, props) || this;
    }
    FlowMapLayer.prototype.initializeState = function () {
        var _a = this.props, getLocationTotalIn = _a.getLocationTotalIn, getLocationTotalOut = _a.getLocationTotalOut, getLocationTotalWithin = _a.getLocationTotalWithin, getLocationId = _a.getLocationId, getLocationCentroid = _a.getLocationCentroid, getFlowOriginId = _a.getFlowOriginId, getFlowDestId = _a.getFlowDestId, getFlowMagnitude = _a.getFlowMagnitude, getFlowColor = _a.getFlowColor;
        var selectors = new Selectors_1.default({
            getLocationId: getLocationId,
            getLocationCentroid: getLocationCentroid,
            getLocationTotalIn: getLocationTotalIn,
            getLocationTotalOut: getLocationTotalOut,
            getLocationTotalWithin: getLocationTotalWithin,
            getFlowOriginId: getFlowOriginId,
            getFlowDestId: getFlowDestId,
            getFlowMagnitude: getFlowMagnitude,
            getFlowColor: getFlowColor,
        });
        this.setState({ selectors: selectors });
    };
    FlowMapLayer.prototype.updateState = function (params) {
        _super.prototype.updateState.call(this, params);
        var props = params.props, changeFlags = params.changeFlags;
        if (changeFlags.propsChanged) {
            var getLocationTotalIn = props.getLocationTotalIn, getLocationTotalOut = props.getLocationTotalOut, getLocationTotalWithin = props.getLocationTotalWithin, getLocationId = props.getLocationId, getLocationCentroid = props.getLocationCentroid, getFlowOriginId = props.getFlowOriginId, getFlowDestId = props.getFlowDestId, getFlowMagnitude = props.getFlowMagnitude, getFlowColor = props.getFlowColor;
            this.state.selectors.setInputAccessors({
                getLocationId: getLocationId,
                getLocationCentroid: getLocationCentroid,
                getLocationTotalIn: getLocationTotalIn,
                getLocationTotalOut: getLocationTotalOut,
                getLocationTotalWithin: getLocationTotalWithin,
                getFlowOriginId: getFlowOriginId,
                getFlowDestId: getFlowDestId,
                getFlowMagnitude: getFlowMagnitude,
                getFlowColor: getFlowColor,
            });
        }
    };
    FlowMapLayer.prototype.getPickingInfo = function (params) {
        var type = getPickType(params.sourceLayer);
        if (!type) {
            return params.info;
        }
        var info = __assign(__assign({}, params.info), { type: type });
        var selectors = this.state.selectors;
        if (type === types_1.PickingType.FLOW) {
            var getLocationById = selectors.getLocationByIdGetter(this.props);
            var _a = selectors.getInputAccessors(), getFlowOriginId = _a.getFlowOriginId, getFlowDestId = _a.getFlowDestId;
            var flow = info.object;
            return __assign(__assign({}, info), (flow && {
                origin: getLocationById(getFlowOriginId(flow)),
                dest: getLocationById(getFlowDestId(flow)),
            }));
        }
        if (type === types_1.PickingType.LOCATION || type === types_1.PickingType.LOCATION_AREA) {
            var location_1 = type === types_1.PickingType.LOCATION ? info.object && info.object.location : info.object;
            var getLocationTotalIn = selectors.getLocationTotalInGetter(this.props);
            var getLocationTotalOut = selectors.getLocationTotalOutGetter(this.props);
            var getLocationTotalWithin = selectors.getLocationTotalWithinGetter(this.props);
            var getLocationCircleRadius = selectors.getLocationCircleRadiusGetter(this.props);
            return __assign(__assign({}, info), (location_1 && {
                object: location_1,
                totalIn: getLocationTotalIn(location_1),
                totalOut: getLocationTotalOut(location_1),
                totalWithin: getLocationTotalWithin(location_1),
                circleRadius: getLocationCircleRadius({ location: location_1, type: "outer" /* OUTER */ }),
            }));
        }
        return info;
    };
    FlowMapLayer.prototype.renderLayers = function () {
        var _a = this.props, showLocationAreas = _a.showLocationAreas, locations = _a.locations, highlightedLocationId = _a.highlightedLocationId;
        var selectors = this.state.selectors;
        var topFlows = selectors.getTopFlows(this.props);
        var highlightedFlows = selectors.getHighlightedFlows(this.props);
        var isLocationHighlighted = highlightedLocationId != null;
        var locationCircles = selectors.getLocationCircles(this.props);
        var layers = [];
        if (showLocationAreas && types_1.isFeatureCollection(locations)) {
            layers.push(this.getLocationAreasLayer(getLayerId(this.props.id, LayerKind.LOCATION_AREAS), false));
        }
        layers.push(this.getFlowLinesLayer(getLayerId(this.props.id, LayerKind.FLOWS), topFlows, false, isLocationHighlighted));
        if (showLocationAreas && types_1.isFeatureCollection(locations)) {
            layers.push(this.getHighlightedLocationAreasLayer(getLayerId(this.props.id, LayerKind.LOCATION_AREAS_SELECTED_AND_HIGHLIGHTED)));
        }
        if (highlightedFlows) {
            layers.push(this.getFlowLinesLayer(getLayerId(this.props.id, LayerKind.FLOWS_HIGHLIGHTED), highlightedFlows, true, false));
        }
        layers.push(this.getLocationCirclesLayer(getLayerId(this.props.id, LayerKind.LOCATIONS), locationCircles, false));
        if (isLocationHighlighted) {
            var highlightedLocationCircles = selectors.getHighlightedLocationCircles(this.props);
            layers.push(this.getLocationCirclesLayer(getLayerId(this.props.id, LayerKind.LOCATIONS_HIGHLIGHTED), highlightedLocationCircles, true));
        }
        if (showLocationAreas && types_1.isFeatureCollection(locations)) {
            layers.push(this.getLocationAreasLayer(getLayerId(this.props.id, LayerKind.LOCATION_AREAS_OUTLINES), true));
        }
        return layers;
    };
    FlowMapLayer.prototype.getLocationAreasLayer = function (id, outline) {
        var _a = this.props, locations = _a.locations, selectedLocationIds = _a.selectedLocationIds, highlightedLocationId = _a.highlightedLocationId, highlightedFlow = _a.highlightedFlow, updateTriggers = _a.updateTriggers;
        var selectors = this.state.selectors;
        var colors = selectors.getColors(this.props);
        return new layers_1.GeoJsonLayer(this.getSubLayerProps(__assign(__assign({ id: id, getFillColor: selectors.getLocationAreaFillColorGetter(this.props), getLineColor: colors.locationAreas.outline, lineJointRounded: true, data: locations, stroked: outline, filled: !outline }, (outline && { pickable: false })), { lineWidthMinPixels: 1, pointRadiusMinPixels: 1, updateTriggers: {
                getFillColor: __assign(__assign({ colors: colors }, (outline && { selectedLocationIds: selectedLocationIds, highlightedLocationId: highlightedLocationId, highlightedFlow: highlightedFlow })), updateTriggers === null || updateTriggers === void 0 ? void 0 : updateTriggers.getLocationAreasFillColor),
                getLineColor: __assign({ colors: colors }, updateTriggers === null || updateTriggers === void 0 ? void 0 : updateTriggers.getLocationAreasLineColor),
            } })));
    };
    FlowMapLayer.prototype.getHighlightedLocationAreasLayer = function (id) {
        var selectors = this.state.selectors;
        var _a = this.props, highlightedLocationId = _a.highlightedLocationId, highlightedLocationAreaId = _a.highlightedLocationAreaId, updateTriggers = _a.updateTriggers;
        var colors = selectors.getColors(this.props);
        var getLocationById = selectors.getLocationByIdGetter(this.props);
        return new layers_1.GeoJsonLayer(this.getSubLayerProps({
            id: id,
            getFillColor: function () { return colors.locationAreas.highlighted; },
            getLineColor: colors.locationAreas.outline,
            lineJointRounded: true,
            data: highlightedLocationId
                ? getLocationById(highlightedLocationId)
                : highlightedLocationAreaId
                    ? getLocationById(highlightedLocationAreaId)
                    : undefined,
            stroked: false,
            filled: true,
            pickable: false,
            lineWidthMinPixels: 5,
            updateTriggers: {
                getFillColor: __assign({ colors: colors }, updateTriggers === null || updateTriggers === void 0 ? void 0 : updateTriggers.getLocationAreasFillColor),
                getLineColor: __assign({ colors: colors }, updateTriggers === null || updateTriggers === void 0 ? void 0 : updateTriggers.getLocationAreasLineColor),
            },
        }));
    };
    FlowMapLayer.prototype.getFlowLinesLayer = function (id, flows, highlighted, dimmed) {
        var _a = this.props, getFlowOriginId = _a.getFlowOriginId, getFlowDestId = _a.getFlowDestId, getFlowMagnitude = _a.getFlowMagnitude, getLocationCentroid = _a.getLocationCentroid, getAnimatedFlowLineStaggering = _a.getAnimatedFlowLineStaggering, showTotals = _a.showTotals, maxLocationCircleSize = _a.maxLocationCircleSize, outlineThickness = _a.outlineThickness, minPickableFlowThickness = _a.minPickableFlowThickness, maxFlowThickness = _a.maxFlowThickness, flowMagnitudeExtent = _a.flowMagnitudeExtent, locationTotalsExtent = _a.locationTotalsExtent, updateTriggers = _a.updateTriggers;
        var selectors = this.state.selectors;
        var endpointOffsets = [(maxLocationCircleSize || 0) + 1, (maxLocationCircleSize || 0) + 1];
        var getLocationRadius = selectors.getLocationCircleRadiusGetter(this.props);
        var getLocationById = selectors.getLocationByIdGetter(this.props);
        var flowThicknessScale = selectors.getFlowThicknessScale(this.props);
        var getSourcePosition = function (flow, info) {
            return getLocationCentroid(getLocationById(getFlowOriginId(flow, info)));
        };
        var getTargetPosition = function (flow, info) {
            return getLocationCentroid(getLocationById(getFlowDestId(flow, info)));
        };
        var getThickness = function (flow, info) { return flowThicknessScale(getFlowMagnitude(flow, info)); };
        var getEndpointOffsets = function (flow, info) {
            if (!showTotals) {
                return endpointOffsets;
            }
            return [
                getLocationRadius({
                    location: getLocationById(getFlowOriginId(flow, info)),
                    type: "outline" /* OUTLINE */,
                }),
                getLocationRadius({
                    location: getLocationById(getFlowDestId(flow, info)),
                    type: "outline" /* OUTLINE */,
                }),
            ];
        };
        var flowColorScale = selectors.getFlowColorScale(this.props);
        var colors = selectors.getColors(this.props);
        var getColor = selectors.getFlowLinesColorGetter(colors, flowColorScale, highlighted, dimmed);
        var animate = this.props.animate;
        var thicknessUnit = maxFlowThickness != null ? maxFlowThickness : FlowLinesLayer_1.default.defaultProps.thicknessUnit;
        var baseProps = __assign(__assign(__assign(__assign(__assign({ id: id,
            getSourcePosition: getSourcePosition,
            getTargetPosition: getTargetPosition,
            getThickness: getThickness,
            getEndpointOffsets: getEndpointOffsets,
            getColor: getColor, data: flows }, (highlighted && { pickable: false })), { drawOutline: !dimmed, updateTriggers: {
                getSourcePosition: updateTriggers === null || updateTriggers === void 0 ? void 0 : updateTriggers.getFlowLinesSourcePosition,
                getTargetPosition: updateTriggers === null || updateTriggers === void 0 ? void 0 : updateTriggers.getFlowLinesTargetPosition,
                getThickness: __assign({ flowMagnitudeExtent: flowMagnitudeExtent,
                    maxFlowThickness: maxFlowThickness }, updateTriggers === null || updateTriggers === void 0 ? void 0 : updateTriggers.getFlowLinesThickness),
                getColor: __assign({ colors: colors,
                    dimmed: dimmed }, updateTriggers === null || updateTriggers === void 0 ? void 0 : updateTriggers.getFlowLinesColor),
                getEndpointOffsets: __assign({ showTotals: showTotals,
                    locationTotalsExtent: locationTotalsExtent }, updateTriggers === null || updateTriggers === void 0 ? void 0 : updateTriggers.getFlowLinesEndpointOffsets),
            }, thicknessUnit: thicknessUnit, outlineColor: colors.outlineColor }), (outlineThickness && { outlineThickness: outlineThickness })), (minPickableFlowThickness != null && {
            getPickable: function (f) { return (thicknessUnit * getThickness(f) >= minPickableFlowThickness ? 1.0 : 0.0); },
        })), { parameters: __assign(__assign({}, this.props.parameters), { depthTest: false }) });
        if (animate) {
            return new AnimatedFlowLinesLayer_1.default(this.getSubLayerProps(__assign(__assign(__assign(__assign({}, baseProps), { currentTime: this.props.animationCurrentTime }), (this.props.animationTailLength != null && {
                animationTailLength: this.props.animationTailLength,
            })), (getAnimatedFlowLineStaggering && {
                getStaggering: getAnimatedFlowLineStaggering,
            }))));
        }
        else {
            return new FlowLinesLayer_1.default(this.getSubLayerProps(baseProps));
        }
    };
    FlowMapLayer.prototype.getLocationCirclesLayer = function (id, circles, highlighted) {
        var _a = this.props, highlightedLocationId = _a.highlightedLocationId, selectedLocationIds = _a.selectedLocationIds, getLocationCentroid = _a.getLocationCentroid, flows = _a.flows, showTotals = _a.showTotals, updateTriggers = _a.updateTriggers, maxLocationCircleSize = _a.maxLocationCircleSize, locationTotalsExtent = _a.locationTotalsExtent;
        var selectors = this.state.selectors;
        var getRadius = showTotals ? selectors.getLocationCircleRadiusGetter(this.props) : function () { return maxLocationCircleSize; };
        var colors = selectors.getColors(this.props);
        var getColor = selectors.getLocationCircleColorGetter(this.props);
        var getPosition = function (locCircle) { return getLocationCentroid(locCircle.location); };
        return new FlowCirclesLayer_1.default(this.getSubLayerProps({
            id: id,
            getColor: getColor,
            getPosition: getPosition,
            getRadius: getRadius,
            data: circles,
            updateTriggers: {
                getRadius: __assign({ showTotals: showTotals,
                    selectedLocationIds: selectedLocationIds,
                    maxLocationCircleSize: maxLocationCircleSize,
                    locationTotalsExtent: locationTotalsExtent,
                    flows: flows }, updateTriggers === null || updateTriggers === void 0 ? void 0 : updateTriggers.getCirclesRadius),
                getColor: __assign({ colors: colors,
                    highlightedLocationId: highlightedLocationId,
                    selectedLocationIds: selectedLocationIds,
                    flows: flows }, updateTriggers === null || updateTriggers === void 0 ? void 0 : updateTriggers.getCirclesColor),
            },
            parameters: __assign(__assign({}, this.props.parameters), { depthTest: false }),
        }));
    };
    FlowMapLayer.layerName = 'FlowMapLayer';
    FlowMapLayer.defaultProps = {
        getLocationId: { type: 'accessor', value: function (l) { return l.id || l.properties.id; } },
        getLocationCentroid: { type: 'accessor', value: function (l) { return l.properties.centroid; } },
        getFlowOriginId: { type: 'accessor', value: function (f) { return f.origin; } },
        getFlowDestId: { type: 'accessor', value: function (f) { return f.dest; } },
        getFlowMagnitude: { type: 'accessor', value: function (f) { return f.count; } },
        showTotals: true,
        maxLocationCircleSize: 15,
        outlineThickness: 1,
        showLocationAreas: true,
        animationTailLength: 0.7,
    };
    return FlowMapLayer;
}(core_1.CompositeLayer));
exports.default = FlowMapLayer;
//# sourceMappingURL=FlowMapLayer.js.map