"use strict";
/*
 * Copyright 2018 Teralytics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFlowColorScale = exports.getDimmedColor = exports.getDimmedCircleColor = exports.getDimmedCircleOutlineColor = exports.opacityFloatToInteger = exports.opacifyHex = exports.rgbaAsString = exports.colorAsRgba = exports.getDiffColorsRGBA = exports.getColorsRGBA = exports.isDiffColorsRGBA = exports.isDiffColors = void 0;
var d3_color_1 = require("d3-color");
var d3_interpolate_1 = require("d3-interpolate");
// @ts-ignore
var d3_scale_1 = require("d3-scale");
var DEFAULT_FLOW_MIN_COLOR = 'rgba(240,240,240,0.5)';
var DEFAULT_FLOW_COLOR_SCHEME = [DEFAULT_FLOW_MIN_COLOR, '#137CBD'];
var DEFAULT_FLOW_COLOR_SCHEME_POSITIVE = [DEFAULT_FLOW_MIN_COLOR, '#f6654e'];
var DEFAULT_FLOW_COLOR_SCHEME_NEGATIVE = [DEFAULT_FLOW_MIN_COLOR, '#00a9cc'];
var DEFAULT_OUTLINE_COLOR = '#fff';
var DEFAULT_LOCATION_AREA_COLOR = 'rgba(220,220,220,0.5)';
var DEFAULT_DIMMED_OPACITY = 0.4;
var CIRCLE_DIMMED_OPACITY_MULTIPLIER = 0.5;
var FALLBACK_COLOR_RGBA = [255, 255, 255, 255];
function isDiffColors(colors) {
    return colors.positive !== undefined;
}
exports.isDiffColors = isDiffColors;
function isDiffColorsRGBA(colors) {
    return colors.positive !== undefined;
}
exports.isDiffColorsRGBA = isDiffColorsRGBA;
function getBaseColorsRGBA(colors) {
    var darkMode = colors && colors.darkMode ? true : false;
    return {
        darkMode: darkMode,
        locationAreas: getLocationAreaColorsRGBA(colors && colors.locationAreas, darkMode),
        outlineColor: colorAsRgba((colors && colors.outlineColor) || DEFAULT_OUTLINE_COLOR),
        dimmedOpacity: colors && colors.dimmedOpacity != null ? colors.dimmedOpacity : DEFAULT_DIMMED_OPACITY,
    };
}
function getColorsRGBA(colors) {
    var baseColorsRGBA = getBaseColorsRGBA(colors);
    return __assign(__assign({}, baseColorsRGBA), getFlowAndCircleColors(colors, DEFAULT_FLOW_COLOR_SCHEME, baseColorsRGBA.darkMode));
}
exports.getColorsRGBA = getColorsRGBA;
function getDiffColorsRGBA(colors) {
    var baseColorsRGBA = getBaseColorsRGBA(colors);
    return __assign(__assign({}, baseColorsRGBA), { positive: getFlowAndCircleColors(colors && colors.positive, DEFAULT_FLOW_COLOR_SCHEME_POSITIVE, baseColorsRGBA.darkMode), negative: getFlowAndCircleColors(colors && colors.negative, DEFAULT_FLOW_COLOR_SCHEME_NEGATIVE, baseColorsRGBA.darkMode) });
}
exports.getDiffColorsRGBA = getDiffColorsRGBA;
function getLocationAreaColorsRGBA(colors, darkMode) {
    var normalColor = (colors && colors.normal) || DEFAULT_LOCATION_AREA_COLOR;
    var normalColorHcl = d3_color_1.hcl(normalColor);
    var locationAreasNormal = colorAsRgba(normalColor);
    return {
        normal: locationAreasNormal,
        connected: colorAsRgbaOr(colors && colors.connected, locationAreasNormal),
        highlighted: colorAsRgbaOr(colors && colors.highlighted, opacifyHex(normalColorHcl[darkMode ? 'brighter' : 'darker'](1).toString(), 0.5)),
        selected: colorAsRgbaOr(colors && colors.selected, opacifyHex(normalColorHcl[darkMode ? 'brighter' : 'darker'](2).toString(), 0.8)),
        outline: colorAsRgbaOr(colors && colors.outline, colorAsRgba(normalColorHcl[darkMode ? 'brighter' : 'darker'](4).toString())),
    };
}
function getFlowAndCircleColors(inputColors, defaultFlowColorScheme, darkMode) {
    var flowColorScheme = (inputColors && inputColors.flows && inputColors.flows.scheme) || defaultFlowColorScheme;
    var maxFlowColorHcl = d3_color_1.hcl(flowColorScheme[flowColorScheme.length - 1]);
    var flowColorHighlighted = colorAsRgbaOr(inputColors && inputColors.flows && inputColors.flows.highlighted, colorAsRgba(maxFlowColorHcl[darkMode ? 'brighter' : 'darker'](0.7).toString()));
    return {
        flows: {
            scheme: flowColorScheme,
            highlighted: flowColorHighlighted,
        },
        locationCircles: {
            inner: colorAsRgbaOr(inputColors && inputColors.locationCircles && inputColors.locationCircles.inner, maxFlowColorHcl.toString()),
            outgoing: colorAsRgbaOr(inputColors && inputColors.locationCircles && inputColors.locationCircles.outgoing, darkMode ? '#000' : '#fff'),
            incoming: colorAsRgbaOr(inputColors && inputColors.locationCircles && inputColors.locationCircles.incoming, maxFlowColorHcl[darkMode ? 'brighter' : 'darker'](1.25).toString()),
            highlighted: colorAsRgbaOr(inputColors && inputColors.locationCircles && inputColors.locationCircles.highlighted, flowColorHighlighted),
        },
    };
}
function colorAsRgba(color) {
    var col = d3_color_1.color(color);
    if (!col) {
        console.warn('Invalid color: ', color);
        return FALLBACK_COLOR_RGBA;
    }
    var rgbColor = col.rgb();
    return [Math.floor(rgbColor.r), Math.floor(rgbColor.g), Math.floor(rgbColor.b), opacityFloatToInteger(col.opacity)];
}
exports.colorAsRgba = colorAsRgba;
function colorAsRgbaOr(color, defaultColor) {
    if (color) {
        return colorAsRgba(color);
    }
    if (typeof defaultColor === 'string') {
        return colorAsRgba(defaultColor);
    }
    return defaultColor;
}
function rgbaAsString(color) {
    return "rgba(" + color.join(',') + ")";
}
exports.rgbaAsString = rgbaAsString;
function opacifyHex(hexCode, opacity) {
    var c = d3_color_1.color(hexCode);
    if (!c) {
        console.warn('Invalid color: ', hexCode);
        return "rgba(255, 255, 255, " + opacity + ")";
    }
    var col = c.rgb();
    return "rgba(" + col.r + ", " + col.g + ", " + col.b + ", " + opacity + ")";
}
exports.opacifyHex = opacifyHex;
function opacityFloatToInteger(opacity) {
    return Math.round(opacity * 255);
}
exports.opacityFloatToInteger = opacityFloatToInteger;
function getDimmedCircleOutlineColor(outlineColor, opacity) {
    var r = outlineColor[0], g = outlineColor[1], b = outlineColor[2], a = outlineColor[3];
    return [r, g, b, a * opacity * CIRCLE_DIMMED_OPACITY_MULTIPLIER];
}
exports.getDimmedCircleOutlineColor = getDimmedCircleOutlineColor;
function getDimmedCircleColor(color, opacity) {
    return getDimmedColor(color, opacity * CIRCLE_DIMMED_OPACITY_MULTIPLIER);
}
exports.getDimmedCircleColor = getDimmedCircleColor;
function getDimmedColor(color, opacity) {
    var col = d3_color_1.hcl(rgbaAsString(color));
    if (!col) {
        console.warn('Invalid color: ', color);
        return FALLBACK_COLOR_RGBA;
    }
    col.c *= 0.1; // desaturate color
    var rgbColor = col.rgb();
    return [
        Math.floor(rgbColor.r),
        Math.floor(rgbColor.g),
        Math.floor(rgbColor.b),
        opacityFloatToInteger(opacity !== undefined ? opacity : DEFAULT_DIMMED_OPACITY),
    ];
}
exports.getDimmedColor = getDimmedColor;
function createFlowColorScale(domain, scheme, animate) {
    var scale = d3_scale_1.scaleSequentialPow(d3_interpolate_1.interpolateRgbBasis(scheme))
        // @ts-ignore
        .exponent(animate ? 1 / 2 : 1 / 3)
        .domain(domain);
    return function (value) { return colorAsRgba(scale(value)); };
}
exports.createFlowColorScale = createFlowColorScale;
//# sourceMappingURL=colors.js.map