import { Feature, FeatureCollection, GeometryObject } from 'geojson';
export declare type Flow = any;
export declare type LocationProperties = any;
export declare type Location = Feature<GeometryObject, LocationProperties> | any;
export declare type Locations = FeatureCollection<GeometryObject, LocationProperties> | Location[];
export declare function isFeatureCollection(locations: Locations): locations is FeatureCollection<GeometryObject, LocationProperties>;
export interface ViewState {
    latitude: number;
    longitude: number;
    zoom: number;
    bearing?: number;
    pitch?: number;
    altitude?: number;
}
export declare const enum LocationCircleType {
    INNER = "inner",
    OUTER = "outer",
    OUTLINE = "outline"
}
export interface LocationCircle {
    location: Location;
    type: LocationCircleType;
}
export interface FlowAccessors {
    getFlowOriginId: FlowAccessor<string>;
    getFlowDestId: FlowAccessor<string>;
    getFlowMagnitude: FlowAccessor<number>;
    getFlowColor?: FlowAccessor<string | undefined>;
    getAnimatedFlowLineStaggering?: FlowAccessor<string | undefined>;
}
export interface LocationAccessors {
    getLocationId: LocationAccessor<string>;
    getLocationCentroid: LocationAccessor<[number, number]>;
    getLocationTotalIn?: LocationAccessor<number>;
    getLocationTotalOut?: LocationAccessor<number>;
    getLocationTotalWithin?: LocationAccessor<number>;
}
export declare type Data = Flow | Location | LocationCircle;
export declare enum PickingType {
    LOCATION = "location",
    FLOW = "flow",
    LOCATION_AREA = "location-area"
}
export declare type DeckGLLayer = any;
export interface PickingInfo<T> {
    layer: DeckGLLayer;
    index: number;
    object: T;
    x: number;
    y: number;
    lngLat: [number, number];
}
export declare type PickingHandler<T> = (info: T, event: {
    srcEvent: MouseEvent;
}) => void;
export interface LocationPickingInfo extends PickingInfo<Data> {
    type: PickingType.LOCATION;
    object: Location;
    totalIn: number;
    totalOut: number;
    totalWithin: number;
    circleRadius: number;
}
export interface LocationAreaPickingInfo extends PickingInfo<Data> {
    type: PickingType.LOCATION_AREA;
    object: Location;
}
export interface FlowPickingInfo extends PickingInfo<Data> {
    type: PickingType.FLOW;
    object: Flow;
    origin: Location;
    dest: Location;
}
export declare type FlowLayerPickingInfo = LocationPickingInfo | LocationAreaPickingInfo | FlowPickingInfo;
export interface AccessorObjectInfo {
    index: number;
    data: any;
    target: any[];
}
export declare type FlowAccessor<T> = (flow: Flow, objectInfo?: AccessorObjectInfo) => T;
export declare type LocationAccessor<T> = (location: Location) => T;
export declare type LocationCircleAccessor<T> = (locCircle: LocationCircle) => T;
export declare type NumberScale = (value: number) => number;
//# sourceMappingURL=types.d.ts.map